# -*- coding: utf-8 -*-
"""
Astro blender by keina
Blender 2.45,2.46,2.47,2.48 / Python 2.5.1,2.5.2 / WindowsXP / Ubuntu810(2.6.27-11-generic)

History:
  2007.07.28(Sat) Beginning
    See Also Reference
    http://hooktail.org/computer/index.php?Gnuplot%A4%C7%A5%D7%A5%E9%A5%CD%A5%BF%A5%EA%A5%A6%A5%E0%A1%AA
    ftp://dbc.nao.ac.jp/DBC/NASAADC/catalogs/5/5050/
  2007.08.12(Sun) 00:33 Finished
  2008.08.02(Sat) 15:52 Summer version
  2008.11.25(Tue) 01:47 Brushup..
  2008.11.25(Tue) 23:12 Usage FileData:AstroDat.py -> AstroDeg.dat
  2009.02.18(Wed) 03:32 Ubuntu810 line:162-163

Usage:
1. Start Blender, go to Text Editor and load three .py files (Astro.py ,
   AstroDat.py , AstroUnit.py). You can load a file from the header menu
   (Text->Open) or the hotkey, [Alt]+[O] on the Text Editor.

2. Select Astro.py file(with the selector, center of the header), then
   Edit following variables and Save it (Text->Save or [Alt]+[S]).

   - FileData: Specify location of the AstroDeg.dat file. (Required)
   - CityIdx: Specify the city where you want to look from by number.
     You can find the number in AstroDat.py,
     but currently it has only cities of Japanese.
   - StarIdx: Specify the asterisms where you want to look at by number
     (See AstroDat.py).

3. Run Astro.py (Text->Run Python Script or [Alt]+[P]). after a while,
   you'll see an sphere of stars in 3D Window.

4. Enjoy!  :-) 


CopyRight(c) 2009 K.Keina
Mail: keines.2007@gmail.com
Web : http://keines.gozaru.jp/blender/
"""
# ***** BEGIN GPL LICENSE BLOCK *****
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import sys,re,math,time
from math import pi,cos,sin,atan,acos,asin,tan,floor,sqrt

from AstroUnit import *
from AstroDat import *

try:
  import Blender
  from Blender import Draw,BGL,Scene,Object,Window,Mesh,Mathutils,Material
  from Blender.Mathutils import Matrix
  IsBlender=True
except:
  IsBlender=False

# + User Config
AstroSize=20
CityIdx=14		# 14:Japan Toyama AstroDat.py
StarIdx=16 		# 16:Orion        AstroDat.py
FileData='/home/keines/blender/Astro/AstroDeg.dat'

# - Time
tm=time.localtime(time.time())
BaseTime=[int(tm[i]) for i in range(6)]
#BaseTime[3]=4 # Direct

JD=JulianDay(BaseTime[0],BaseTime[1],BaseTime[2])

# - rad
d24=(2*pi)/24.0
d36=(2*pi)/360.0

# + Base Name
BaseName=['Star.Line','Star','Empty']
BaseNameLine=0
BaseNameStar=1
BaseNameEmpty=2

# Longitude WN to WE
def ConvWest(s,v):
  if re.search("[wW]", s):
    v*=-1
  elif re.search("[eE]", s):
    v=360-v
  return v

def AstroMake(f,TM,Lo,la):
  # + Preferences
  L=rdAngle(Lo)		# dms:Longitude
  l=rdAngle(la)		# dms:Latitude
  LW=ConvWest(Lo,L)
  work='-'
  # + Stellar data
  f = file(f)
  f.readline()
  vList={}
  rexp=re.compile("[^\t\n]+")
  for i,w in enumerate(f):
    w=rexp.findall(w)
    r=float(w[0])  # RA
    d=float(w[1])  # DE
    # + r,d decimal->rad
    x,y,z,A,a=horizontal(TM,JD,r*d24,d*d36,LW,l,TM[3],TM[4],TM[5])
    x*=-AstroSize
    y*=AstroSize
    z*=AstroSize
    #if z>-1: # Ground Under..
    if not vList.has_key(w[2]): vList[w[2]]={}			# Vmag
    if not vList[w[2]].has_key(w[3]): vList[w[2]][w[3]]=[]	# Spec
    vList[w[2]][w[3]].append([x,y,z])
  if not IsBlender:return
  # + Make Object
  editmode = Window.EditMode()
  if editmode: Window.EditMode(0)
  # value
  for v1 in vList.iteritems():
    # color
    for v2 in v1[1].iteritems():
      # Have Object ?
      IsBool=True
      name=BaseName[BaseNameStar]+'['+v1[0]+'_'+v2[0]+']'
      try:
        ob = Blender.Object.Get(name)
        me=ob.getData(mesh=1)
        if me!=None:
	  # once reset..
          me.verts=None
          IsBool=not IsBool
      except:
        None
      if IsBool:
        me = Mesh.New(name)
        scn = Scene.GetCurrent()
        ob = scn.objects.new(me, name) # Vmag_Spec
      
      me.verts.extend(v2[1])
      
      # + Material
      try:
        mat=Material.Get(ob.name)
      except:
        mat = Material.New(ob.name)
      # + spec ABCFGKMNOpSW
      #mat.rgbCol=Spec2RGB(v2[0]) # ubuntu810 :oh no..
      mat.rgbCol=Spec2RGB(v2[0][0])
      
      # Material Property
      v=float(v1[0])
      mat.mode |= Material.Modes.HALO
      mat.haloSize=Vmag2Size(v)
      me.materials=[mat]
      me.update()
  
  Blender.Window.RedrawAll()
  if editmode: Window.EditMode(1)
  
  #Blender.Window.DrawProgressBar(1, '')

# right ascension:0..24   celestial declination:-90..90
def LineMake(TM,Lo,la):
  # + Preferences
  v=[]			# Vertexes
  e=[]			# Edge
  n={}			# Right Ascension Index
  k=0			# Count
  RA=24
  DE=9
  DEd=10
  L=rdAngle(Lo)		# dms:Longitude sexagesimal number
  l=rdAngle(la)		# dms:Latitude  sexagesimal number
  LW=ConvWest(Lo,L)
  
  # + line edge
  #   - Right ascension loop
  for ra in range(RA) :			 # RA0..23
    # - Declination loop
    IsAppend=False
    for dc in range(-8,9) : 		# -8, 0..18
      r=rdAngle(str(ra)+' 00 00')	# hms:Right ascension
      d=rdAngle(str(dc*DEd)+' 00 00')	# dms:Declination
      #   - r,d 24,36 rad
      x,y,z,A,a=horizontal(TM,JD,r*d24,d*d36,LW,l,TM[3],TM[4],TM[5])
      x*=-AstroSize
      y*=AstroSize
      z*=AstroSize
      #if z>=0:				# Underground
      IsAppend=True
      v.append([x,y,z])			# Vertex
      e.append([k,k+1])			# Declination Edge
      n[ra,dc]=len(v)
      k+=1
    if IsAppend: del e[len(e)-1]	# Last Edge
  if not IsBlender : return
  
  # + Right ascension edge
  w=[0,0]
  for r in range(RA-2) :		# RA
    for d in range(-8,9):		# -8,9
      if n.has_key((r,d)):
        if n.has_key((r+1,d)):
	    e.append([n[r,d] ,n[r+1,d]])
  #   - first and last
  r=RA-2
  for d in range(-8,9):
    if n.has_key((0,d)):
      if n.has_key((r,d)):
        e.append([n[0,d] ,n[r,d]])
  # + Make Object(Edge)
  editmode = Window.EditMode()
  if editmode: Window.EditMode(0)
  
  # Have Object ?
  IsBool=True
  try:
    ob = Blender.Object.Get(BaseName[BaseNameLine])
    me=ob.getData(mesh=1)
    if me!=None:
      me.verts=None
      IsBool=not IsBool
  except:
    None
  
  if IsBool:
    me = Mesh.New(BaseName[BaseNameLine])
    scn = Scene.GetCurrent()
    ob = scn.objects.new(me, BaseName[BaseNameLine])
  
  me.verts.extend(v)
  me.faces.extend(e)
  
  # Have Material ?
  try:
    mat=Material.Get(BaseName[BaseNameLine])
  except:
    mat = Material.New(ob.name)
  # Material Property
  mat.mode |= Material.Modes.WIRE | Material.Modes.ZTRANSP | Material.Modes.SHADELESS
  mat.alpha = 0.65
  mat.emit = 1.0
  mat.rgbCol=[0.393,0.690,1.0]
  me.materials=[mat]
  me.update()
  
  Blender.Window.RedrawAll()
  if editmode: Window.EditMode(1)
  #Blender.Window.DrawProgressBar(1, '')

# 3D Cursor
def Indicate(TM,ra,dc,Lo,la): # lon,lat,dat,h,m,s
  # + Preferences
  #   - Converted to decimal
  L=rdAngle(Lo)		# dms Longitude
  l=rdAngle(la)		# dms Latitude
  r=rdAngle(ra)		# hms Right ascension
  d=rdAngle(dc)		# dms Declination
  LW=ConvWest(Lo,L)
  # + Coordinate transformation
  x,y,z,A,a=horizontal(TM,JD,r*d24,d*d36,LW,l,TM[3],TM[4],TM[5])
  x*=-AstroSize
  y*=AstroSize
  z*=AstroSize
  # + Move 3D Cursor
  Blender.Window.SetCursorPos(x,y,z)
  try:
    ob=Object.Get(BaseName[BaseNameEmpty])
    ob.setLocation (x, y, z)
  except:
    None
  Window.Redraw(Window.Types["VIEW3D"])
  print "Astro : %s -> 3D Cursor %s" % (CityDat[CityIdx][0],StarDat[StarIdx][0])

# Star
AstroMake(FileData,BaseTime,CityDat[CityIdx][1],CityDat[CityIdx][2])

# Edge Longitude Latitude
#LineMake(BaseTime,CityDat[CityIdx][1],CityDat[CityIdx][2])

# 3D Cursor
Indicate(BaseTime,StarDat[StarIdx][1],StarDat[StarIdx][2],CityDat[CityIdx][1],CityDat[CityIdx][2])